// ForGet. A tool for managing development dependencies.
// Copyright (C) 2015  Daniel Hughes
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Xml.Linq;
using System.Collections.Generic;
using System.IO;

namespace ForGet
{
   public static class ConfigHelpers
   {
      public static IEnumerable<string> GetRepoPaths(XElement repo)
      {
         string platform = Environment.OSVersion.Platform == PlatformID.Unix ? 
            "Linux" :
            "Windows";
                 
         foreach(var source in repo.Elements("Source"))
         {
            string repoPath = source.Attribute(platform).Value;
            if(!Directory.Exists(repoPath))
            {
               Console.Error.WriteLine("Error: Repo path could not be found: {0}", repoPath);
               yield return null;
            }
            yield return repoPath;
         }
      }

        public static IPackageSource GetPackageSource(XElement repo)
        {
            string platform = Environment.OSVersion.Platform == PlatformID.Unix ?
                "Linux" :
                "Windows";

            var packageSources = new List<IPackageSource>();
            foreach (var source in repo.Elements("Source"))
            {
                string repoPath = source.Attribute(platform).Value;
                if(repoPath.StartsWith("https://") || repoPath.StartsWith("http://"))
                {
                    packageSources.Add(new WebApiRespositoryV2(repoPath));
                    continue;
                }
                if (!Directory.Exists(repoPath))
                {
                    Console.Error.WriteLine("Error: Repo path could not be found: {0}", repoPath);
                    return null;
                }
                packageSources.Add(new FileShareRepository(repoPath));
            }

            return new MultipleSourceRepository(packageSources);
        }
   }
}

