﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace ForGet
{
    public class FileShareRepository : IPackageSource
    {
        static readonly string[] _supportedPackageExtenstions = new[] { "forpkg", "nupkg" };

        readonly string _repo;

        public FileShareRepository(string repo)
        {
            _repo = repo;
        }

        public IEnumerable<string> GetPackagesById(string id)
        {
            var repoPathforDependency = Path.Combine(_repo, id);
            if(!Directory.Exists(repoPathforDependency))
            {
                return Enumerable.Empty<string>();
            }

             return
                from extenstion in _supportedPackageExtenstions
                from filePath in Directory.GetFiles(repoPathforDependency, string.Format("*.{0}", extenstion), SearchOption.AllDirectories)
                select filePath;
        }

        public string GetPackage(string id, string version)
        {
            var matches =
               from package in GetPackagesById(id)
               let packageNameWithoutExtension = Path.GetFileNameWithoutExtension(package)
               where IsPackageMatch(package, id, version)
               select package;

            return matches.FirstOrDefault();
        }

        bool IsPackageMatch(string filename, string id, string version)
        {
            filename = Path.GetFileNameWithoutExtension(filename).ToLowerInvariant();

            if (!filename.StartsWith(id.ToLowerInvariant()))
                return false;

            Version expectedVersion;
            if (!TryParseVersion(version, out expectedVersion))
                return false;

            Version fileVersion;
            if (!TryParseVersion(filename.Substring(id.Length + 1), out fileVersion))
                return false;

            return fileVersion == expectedVersion;
        }

        bool TryParseVersion(string version, out Version result)
        {
            while (version.Count(c => c == '.') < 3)
            {
                version = version + ".0";
            }

            return Version.TryParse(version, out result);
        }

        public IEnumerable<Version> GetVersions(string packageId)
        {
            return
                from package in GetPackagesById(packageId)
                select GetVersion(packageId, package);
        }

        Version GetVersion(string id, string file)
        {
            string fileName = Path.GetFileNameWithoutExtension(file);
            string versionPart = fileName.Substring(id.Length + 1);
            Version version = null;
            if (!Version.TryParse(versionPart, out version))
            {
                return null;
            }
            return version;
        }

        public string DownloadPackage(string packageId, string version)
        {
            return GetPackage(packageId, version);
        }
    }
}
