﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Xml.Linq;

namespace ForGet
{
    public class WebApiRespositoryV2 : IPackageSource
    {
        readonly Uri _repoUrl;

        public WebApiRespositoryV2(string repoUrl)
        {
            _repoUrl = new Uri(repoUrl);
        }

        public string DownloadPackage(string packageId, string version)
        {
            string query = $"Packages(Id={packageId},Version={version})/Download"; 
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            var uri = new Uri(_repoUrl, query);
            var packagesResult = client.GetAsync(uri).Result;
            if (!packagesResult.IsSuccessStatusCode)
            {
                Console.Error.WriteLine($"Failed to get package {packageId} with StatusCode: {packagesResult.StatusCode} from: {_repoUrl}");
                return null;
            }
            var downloaded = packagesResult.Content.ReadAsByteArrayAsync().Result;
            var tempFile = Path.GetTempFileName();
            File.WriteAllBytes(tempFile, downloaded);
            return tempFile;
        }

        public IEnumerable<Version> GetVersions(string packageId)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            var uri = new Uri(_repoUrl, $"FindPackagesById()?id=\'{packageId}\'");
            var packagesResult = client.GetAsync(uri).Result;
            if (!packagesResult.IsSuccessStatusCode)
            {
                Console.Error.WriteLine($"Failed to get packages for {packageId} with StatusCode: {packagesResult.StatusCode} from: {_repoUrl}");
                return null;
            }
            var xml = packagesResult.Content.ReadAsStringAsync().Result;
            var doc = XDocument.Parse(xml);
            var versions =
                from entry in doc.Root.ElementsLocalName("entry")
                let versionString = entry.ElementLocalName("properties").ElementLocalName("Version").Value
                select Version.Parse(versionString);

            return versions;
        }
    }
}
